==> pathena-0.3.3.tgz: .pathena/lib/platform_utilities.py <==
# OS-specific operations are defined in this module.  Mostly these
# are used to invoke commands or external programs with various
# combinations of shell controls.  Some GUI-based operations are
# also included.

# Currently these features are limited to Unix family execution
# environments.  They have been tested with Linux.  They might need
# adjustment for BSD platforms.


import os, re, signal, thread, Queue

from app_globals       import *
from logging_services  import *

from preference_utilities import user_prefs


command_timeout = user_prefs.getint('Application', 'command_timeout')

max_indexed_len = user_prefs.getint('Indexing', 'max_indexed_len')

head_cmd = 'head -c %s' % max_indexed_len

cmd_trunc_len = max_indexed_len


def escaped_blanks(str, n):
    for i in range(n):
        str = str.replace(' ', '\ ')   # protect file names in shell commands
    return str

def quote_wrap(str, quote_null=0):
    if not (str or quote_null) or str.startswith('"') and str.endswith('"'):
        return str
    return '"%s"' % str


# Following functions are used throughout the application to invoke
# external OS commands and standard utility programs.

def make_shell_command(command, timeout=None):
    # command is of form: 'cmd %s'  # path_name
    # stdout output returned as string, stderr output discarded
    # timeout in seconds to abort
    def invoke_command(path_name=None):
        # change later to popen3/4???
        # invoke command, protect path name with quotes
        if path_name: cmd = command % ('"%s"' % path_name)
        else:         cmd = command
        ### might want to save error output in a log file
        pipe = os.popen('(%s) 2> /dev/null' % cmd)
        output = pipe.read()
        exit_code = pipe.close()
        if exit_code: return ''   # add reporting or other actions later
        return output
    return invoke_command
            
def shell_command_with_err(command):
    # command is of form: 'cmd %s'  # path_name
    # output returned as string, including both stdout and stderr 
    def invoke_command(path_name=None):
        # invoke command, protect path name with quotes
        if path_name: cmd = command % ('"%s"' % path_name)
        else:         cmd = command
        pipe_to, pipe_from_err = os.popen4(cmd)
        pipe_to.close()
        output = pipe_from_err.read()
        pipe_from_err.close()
        return output
    return invoke_command


# Following provides a protected way to invoke user supplied shell
# commands.  The shell /bin/sh is spawned explicitly using a
# specially constructed command string.  Returns pid and pipe
# for command shell process unless command is ill-formed, in which
# case a message is returned.

### consider adding command output truncation

def shell_command_with_timeout(command_str, timeout=command_timeout):
    # command is of form: 'cmd %s'  # path_name
    # output returned as string, including both stdout and stderr 
    def invoke_command(path_name=None):
        command = command_str.strip()
        if not command: return ''
        # invoke command, protect path name with quotes
        num_sites = command.count('%s')
        if path_name and num_sites == 0:
            return "***** Try again: insert %s into your command."
        if not path_name and num_sites > 0:
            return "***** No path name(s) to substitute for %s."
        if path_name:
            cmd = command % ((quote_wrap(path_name),) * num_sites)
        else:
            cmd = command
        if cmd[0] == '(' and cmd[-1] == ')': cmd = cmd[1:-1]
        # outer quotes ('..') needed so spawn script keeps intact as one arg
        # pipe to cat gives exit code 0 and preserves shell error messages
        cmd = "'(%s) | cat'" % cmd.replace('"', r'\"')
        return spawn_timeout_pipe('/bin/sh', '-c', cmd, timeout, '', 1, 2)()
    return invoke_command

# Spawn exec_file on args in left_args + "path" + right_args with a
# limit of timeout seconds.  All spaces in left_args and right_args
# are taken to be interword spaces as interpreted by the shell.
# Direct stdout and stderr with values: None - discard,
# 1/2 - return as string, else - file for redirection.
# Default: return stdout as a string; discard stderr.
# Returns command pid (in case interrupt needed) and command in, out pipes.

def spawn_timeout_pipe(exec_file, left_args, right_args, timeout='',
                       trunc_len='', stdout=1, stderr=None):
    def invoke_command(path_name=''):
        # construct pipe command, protect args with quotes
        spawn_cmd = '%s %s %s %s %s %s %s' % \
                    (spawn_bin, exec_file, quote_wrap(left_args, 1),
                     quote_wrap(path_name, 1),
                     quote_wrap(right_args, 1), timeout, trunc_len)
        if stdout == None: redirect_o = ' > /dev/null'
        elif stdout == 1:  redirect_o = ''
        else:              redirect_o = ' > ' + stdout
        if stderr == None: redirect_e = ' 2> /dev/null'
        elif stderr == 2:  redirect_e = ' 2>&1'
        else:              redirect_e = ' 2> ' + stderr
        full_cmd = '%s%s%s' % (spawn_cmd, redirect_o, redirect_e)
        cmd_in, cmd_out, cmd_err = os.popen3(full_cmd)
        cmd_pid = int(cmd_out.readline())      # should respond immediately
        queue = Queue.Queue()
        thread.start_new_thread(listen_for_cmd_out, (queue, cmd_in, cmd_out))
        return cmd_pid, queue
    return invoke_command

# A separate listener thread is used to wait for command output,
# then close both pipes.

def listen_for_cmd_out(queue, cmd_in, cmd_out):
    queue.put(cmd_out.read())
    cmd_out.close()
    cmd_in.close()

# Use one of these two functions to finish up spawned command.
# Protocol is that last 6 chars of command output is the exit code.
# Raises CommandTimeout if timeout or CommandAbort if user aborted or
# CommandFailure if other error detected.

def parse_cmd_out(cmd_output):
    exit_code = int(cmd_output[-6:])
    if   exit_code == 1: raise CommandTimeout, 'Command timed out'
    elif exit_code == 2: raise CommandAbort, 'Command interrupted by user'
    elif exit_code  > 8:
        raise CommandFailure, \
              'Command failed with exit code %s' % (exit_code - 8)
    return cmd_output[:-6]     # lop off exit code

def abort_spawned_cmd(cmd_pid):
    try:
        os.kill(cmd_pid, signal.SIGTERM)
    except OSError:
        pass        # process no longer exists


# Spawn a process and truncate output to an initial prefix.  Assume
# that exec_file needs at most one instance of path name.

def truncated_spawn(exec_file, left_args, right_args, timeout='',
                    stdout=1, stderr=None):
    def invoke_cmd(path_name=''):
        # enable pipefail option to trigger error if any pipe segment does
        cmd = 'set -o pipefail; %s %s %s %s' % \
              (exec_file, left_args, quote_wrap(path_name), right_args)
        # outer quotes ('..') needed so spawn script keeps intact as one arg
        cmd = "'%s'" % cmd.replace('"', r'\"')
        pid, queue = \
             spawn_timeout_pipe('/bin/sh', '-c', cmd, timeout,
                                cmd_trunc_len, stdout, stderr)()
        return parse_cmd_out(queue.get()) # can raise one of several exceptions
    return invoke_cmd

    
def check_os_command(command):
    # check for existence of shell command/program; returns exit code
    pipe = os.popen('which %s > /dev/null 2> /dev/null' % command)
    return pipe.close()

def extract_directory_children(options):
    return make_shell_command('ls %s %%s' % options) #, redirect=1)

extract_directory_children_short = extract_directory_children('')
extract_directory_children_long  = extract_directory_children('-l')

_link_target_pattern = re.compile('.*->(.*)')  # match output of ls command
_ls_cmd = make_shell_command('ls -l %s')

def extract_link_target(path_name):
    target = _link_target_pattern.match(_ls_cmd(path_name)).group(1)
    return target
#    return target.replace('/', ' ')


# Check if a file contains either binary data or printable text
# using OS 'file' command (-b option is for brief output).

_file_cmd = make_shell_command('file -b %s')

def look_up_file_attribs(path):
    return _file_cmd(path)

def contains_binary(path):
    attribs = look_up_file_attribs(path)
    return 'executable' in attribs and 'text' not in attribs

def contains_text(path):
    return 'text' in look_up_file_attribs(path)

### Following should try to identify if binary, extract whatever text it can.
### Does nothing for now.

def extract_binary_text(path_name):
    return 'This is a binary executable file.'


#---------------

### Following are GUI-related.

# Shell commands to invoke external terminal windows (shells).
# Each entry is of form:  (<display_name>, <start-cmd>, <exec-option>)
# The <exec-option> is '-e' or '-x' or equivalent so that a terminal
# can be started with a shell command such as:
#   <start-cmd> <exec-option> <interior-cmd> <arg 1> ...
# Common terminal invocations for standard shells (sh, bash, tcsh):
#     gnome-terminal -x <shell> -s <arg 1> <arg 2> ...
#     konsole -e <shell> -s <arg 1> <arg 2> ...
#     xterm   -e <shell> -s <arg 1> <arg 2> ...

terminal_windows = (
    ('Gnome Terminal', 'gnome-terminal', '-x'),
    ('KDE Konsole', 'konsole', '-e'),
    ('X Terminal (xterm)', 'xterm', '-e'),
    )


# Shell commands to invoke external editors for text files.
# Each entry is of the form:  (<display_name>, <command>)
# Commands contain %s where path name of file is required.

text_editors = (
    ('Emacs', 'emacs %s'),
    ('XEmacs', 'xemacs %s'),
    ('Emacsclient', 'emacsclient -n %s'),
    ('Vim', 'gvim %s'),
    )

==> pathena-0.3.4.tgz: .pathena/lib/platform_utilities.py <==
# OS-specific operations are defined in this module.  Mostly these
# are used to invoke commands or external programs with various
# combinations of shell controls.  Some GUI-based operations are
# also included.

# Currently these features are limited to Unix family execution
# environments.  They have been tested with Linux.  They might need
# adjustment for BSD platforms.


import os, re, signal, thread, Queue

from app_globals       import *
from logging_services  import *

from preference_utilities import user_prefs


command_timeout = user_prefs.getint('Application', 'command_timeout')

max_indexed_len = user_prefs.getint('Indexing', 'max_indexed_len')

head_cmd = 'head -c %s' % max_indexed_len

cmd_trunc_len = max_indexed_len


def escaped_blanks(str, n):
    for i in range(n):
        str = str.replace(' ', '\ ')   # protect file names in shell commands
    return str

def quote_wrap(str, quote_null=0):
    if not (str or quote_null) or str.startswith('"') and str.endswith('"'):
        return str
    return '"%s"' % str


# Following functions are used throughout the application to invoke
# external OS commands and standard utility programs.

def make_shell_command(command, timeout=None):
    # command is of form: 'cmd %s'  # path_name
    # stdout output returned as string, stderr output discarded
    # timeout in seconds to abort
    def invoke_command(path_name=None):
        # change later to popen3/4???
        # invoke command, protect path name with quotes
        if path_name: cmd = command % ('"%s"' % path_name)
        else:         cmd = command
        ### might want to save error output in a log file
        pipe = os.popen('(%s) 2> /dev/null' % cmd)
        output = pipe.read()
        exit_code = pipe.close()
        if exit_code: return ''   # add reporting or other actions later
        return output
    return invoke_command
            
def shell_command_with_err(command):
    # command is of form: 'cmd %s'  # path_name
    # output returned as string, including both stdout and stderr 
    def invoke_command(path_name=None):
        # invoke command, protect path name with quotes
        if path_name: cmd = command % ('"%s"' % path_name)
        else:         cmd = command
        pipe_to, pipe_from_err = os.popen4(cmd)
        pipe_to.close()
        output = pipe_from_err.read()
        pipe_from_err.close()
        return output
    return invoke_command


# Following provides a protected way to invoke user supplied shell
# commands.  The shell /bin/sh is spawned explicitly using a
# specially constructed command string.  Returns pid and pipe
# for command shell process unless command is ill-formed, in which
# case a message is returned.

### consider adding command output truncation

def shell_command_with_timeout(command_str, timeout=command_timeout):
    # command is of form: 'cmd %s'  # path_name
    # output returned as string, including both stdout and stderr 
    def invoke_command(path_name=None):
        command = command_str.strip()
        if not command: return ''
        # invoke command, protect path name with quotes
        num_sites = command.count('%s')
        if path_name and num_sites == 0:
            return "***** Try again: insert %s into your command."
        if not path_name and num_sites > 0:
            return "***** No path name(s) to substitute for %s."
        if path_name:
            cmd = command % ((quote_wrap(path_name),) * num_sites)
        else:
            cmd = command
        if cmd[0] == '(' and cmd[-1] == ')': cmd = cmd[1:-1]
        # outer quotes ('..') needed so spawn script keeps intact as one arg
        # pipe to cat gives exit code 0 and preserves shell error messages
        cmd = "'(%s) | cat'" % cmd.replace('"', r'\"')
        return spawn_timeout_pipe('/bin/sh', '-c', cmd, timeout, '', 1, 2)()
    return invoke_command

# Spawn exec_file on args in left_args + "path" + right_args with a
# limit of timeout seconds.  All spaces in left_args and right_args
# are taken to be interword spaces as interpreted by the shell.
# Direct stdout and stderr with values: None - discard,
# 1/2 - return as string, else - file for redirection.
# Default: return stdout as a string; discard stderr.
# Returns command pid (in case interrupt needed) and command in, out pipes.

def spawn_timeout_pipe(exec_file, left_args, right_args, timeout='',
                       trunc_len='', stdout=1, stderr=None):
    def invoke_command(path_name=''):
        # construct pipe command, protect args with quotes
        spawn_cmd = '%s %s %s %s %s %s %s' % \
                    (spawn_bin, exec_file, quote_wrap(left_args, 1),
                     quote_wrap(path_name, 1),
                     quote_wrap(right_args, 1), timeout, trunc_len)
        if stdout == None: redirect_o = ' > /dev/null'
        elif stdout == 1:  redirect_o = ''
        else:              redirect_o = ' > ' + stdout
        if stderr == None: redirect_e = ' 2> /dev/null'
        elif stderr == 2:  redirect_e = ' 2>&1'
        else:              redirect_e = ' 2> ' + stderr
        full_cmd = '%s%s%s' % (spawn_cmd, redirect_o, redirect_e)
        cmd_in, cmd_out, cmd_err = os.popen3(full_cmd)
        cmd_pid = int(cmd_out.readline())      # should respond immediately
        queue = Queue.Queue()
        thread.start_new_thread(listen_for_cmd_out, (queue, cmd_in, cmd_out))
        return cmd_pid, queue
    return invoke_command

# A separate listener thread is used to wait for command output,
# then close both pipes.

def listen_for_cmd_out(queue, cmd_in, cmd_out):
    queue.put(cmd_out.read())
    cmd_out.close()
    cmd_in.close()

# Use one of these two functions to finish up spawned command.
# Protocol is that last 6 chars of command output is the exit code.
# Raises CommandTimeout if timeout or CommandAbort if user aborted or
# CommandFailure if other error detected.

def parse_cmd_out(cmd_output):
    exit_code = int(cmd_output[-6:])
    if   exit_code == 1: raise CommandTimeout, 'Command timed out'
    elif exit_code == 2: raise CommandAbort, 'Command interrupted by user'
    elif exit_code  > 8:
        raise CommandFailure, \
              'Command failed with exit code %s' % (exit_code - 8)
    return cmd_output[:-6]     # lop off exit code

def abort_spawned_cmd(cmd_pid):
    try:
        os.kill(cmd_pid, signal.SIGTERM)
    except OSError:
        pass        # process no longer exists


# Spawn a process and truncate output to an initial prefix.  Assume
# that exec_file needs at most one instance of path name.

def truncated_spawn(exec_file, left_args, right_args, timeout='',
                    stdout=1, stderr=None):
    def invoke_cmd(path_name=''):
        # enable pipefail option to trigger error if any pipe segment does
        cmd = 'set -o pipefail; %s %s %s %s' % \
              (exec_file, left_args, quote_wrap(path_name), right_args)
        # outer quotes ('..') needed so spawn script keeps intact as one arg
        cmd = "'%s'" % cmd.replace('"', r'\"')
        pid, queue = \
             spawn_timeout_pipe('/bin/sh', '-c', cmd, timeout,
                                cmd_trunc_len, stdout, stderr)()
        return parse_cmd_out(queue.get()) # can raise one of several exceptions
    return invoke_cmd

    
def check_os_command(command):
    # check for existence of shell command/program; returns exit code
    pipe = os.popen('which %s > /dev/null 2> /dev/null' % command)
    return pipe.close()

def extract_directory_children(options):
    return make_shell_command('ls %s %%s' % options) #, redirect=1)

extract_directory_children_short = extract_directory_children('')
extract_directory_children_long  = extract_directory_children('-l')

_link_target_pattern = re.compile('.*->(.*)')  # match output of ls command
_ls_cmd = make_shell_command('ls -l %s')

def extract_link_target(path_name):
    target = _link_target_pattern.match(_ls_cmd(path_name)).group(1)
    return target
#    return target.replace('/', ' ')


# Check if a file contains either binary data or printable text
# using OS 'file' command (-b option is for brief output).

_file_cmd = make_shell_command('file -b %s')

def look_up_file_attribs(path):
    return _file_cmd(path)          # returns a string

def contains_binary(path):
    attribs = look_up_file_attribs(path)
    return attribs.find('executable') >= 0 and attribs.find('text') < 0
#    return 'executable' in attribs and 'text' not in attribs    ### 2.3

def contains_text(path):
    return look_up_file_attribs(path).find('text') >= 0
#    return 'text' in look_up_file_attribs(path)                 ### 2.3

### Following should try to identify if binary, extract whatever text it can.
### Does nothing for now.

def extract_binary_text(path_name):
    return 'This is a binary executable file.'


#---------------

### Following are GUI-related.

# Shell commands to invoke external terminal windows (shells).
# Each entry is of form:  (<display_name>, <start-cmd>, <exec-option>)
# The <exec-option> is '-e' or '-x' or equivalent so that a terminal
# can be started with a shell command such as:
#   <start-cmd> <exec-option> <interior-cmd> <arg 1> ...
# Common terminal invocations for standard shells (sh, bash, tcsh):
#     gnome-terminal -x <shell> -s <arg 1> <arg 2> ...
#     konsole -e <shell> -s <arg 1> <arg 2> ...
#     xterm   -e <shell> -s <arg 1> <arg 2> ...

terminal_windows = (
    ('Gnome Terminal', 'gnome-terminal', '-x'),
    ('KDE Konsole', 'konsole', '-e'),
    ('X Terminal (xterm)', 'xterm', '-e'),
    )


# Shell commands to invoke external editors for text files.
# Each entry is of the form:  (<display_name>, <command>)
# Commands contain %s where path name of file is required.

text_editors = (
    ('Emacs', 'emacs %s'),
    ('XEmacs', 'xemacs %s'),
    ('Emacsclient', 'emacsclient -n %s'),
    ('Vim', 'gvim %s'),
    )
